@@ -23,12 +23,15 @@ import jax.numpy as jnp
 import matplotlib
 import matplotlib.pyplot as plt
 import numpy as np
+import pandas as pd
 import seaborn as sns
 from sklearn import metrics
 
 from lightweight_mmm import lightweight_mmm
 from lightweight_mmm import preprocessing
 
+_PALETTE = sns.color_palette(n_colors=100)
+
 
 @functools.partial(jax.jit, static_argnames=("media_mix_model"))
 def _make_single_prediction(media_mix_model: lightweight_mmm.LightweightMMM,
@@ -129,6 +132,129 @@ def _calculate_number_rows_plot(n_media_channels: int, n_columns: int):
   return n_media_channels // n_columns + 2
 
 
+def create_media_baseline_contribution_df(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    target_scaler: Optional[preprocessing.CustomScaler] = None,
+    channel_names: Optional[Sequence[str]] = None) -> pd.DataFrame:
+  """Creates a dataframe for weekly media channels & basline contribution.
+
+  The output dataframe will be used to create a stacked area plot to visualize
+  the contribution of each media channels & baseline.
+
+  Args:
+    media_mix_model: Media mix model.
+    target_scaler: Scaler used for scaling the target.
+    channel_names: Names of media channels.
+
+  Returns:
+    contribution_df: DataFrame of weekly channels & baseline contribution
+    percentage & volume.
+
+  Raises:
+    NotFittedModelError: If the model is not fitted before visualization.
+  """
+  if not hasattr(media_mix_model, "trace"):
+    raise lightweight_mmm.NotFittedModelError(
+        "Model needs to be fit first before attempting to plot its fit.")
+
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    # s for samples, t for time, c for media channels, g for geo
+    einsum_str = "stcg, scg->stcg"
+  elif media_mix_model.trace["media_transformed"].ndim == 3:
+    # s for samples, t for time, c for media channels
+    einsum_str = "stc, sc->stc"
+
+  scaled_media_contribution = jnp.einsum(
+      einsum_str, media_mix_model.trace["media_transformed"],
+      media_mix_model.trace["coef_media"])
+
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    # Aggregate media channel contribution across geos.
+    scaled_media_contribution = scaled_media_contribution.sum(axis=-1)
+
+  # Aggregate media channel contribution across samples.
+  sum_scaled_media_contribution_across_samples = scaled_media_contribution.sum(
+      axis=0)
+  # Aggregate media channel contribution across channels.
+  sum_scaled_media_contribution_across_channels = scaled_media_contribution.sum(
+      axis=2)
+
+  # Calculate the baseline contribution.
+  # Scaled prediction - sum of scaled contribution across channels.
+  scaled_prediction = media_mix_model.trace["mu"]
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    # Sum up the scaled prediction across all the geos.
+    scaled_prediction = scaled_prediction.sum(axis=-1)
+  baseline_contribution = scaled_prediction - sum_scaled_media_contribution_across_channels
+
+  # Sum up the scaled media, baseline contribution and predictio across samples.
+  sum_scaled_media_contribution_across_channels_samples = sum_scaled_media_contribution_across_channels.sum(
+      axis=0)
+  sum_scaled_baseline_contribution_across_samples = baseline_contribution.sum(
+      axis=0)
+
+  # Adjust baseline contribution and prediction when there's any negative value.
+  adjusted_sum_scaled_baseline_contribution_across_samples = np.where(
+      sum_scaled_baseline_contribution_across_samples < 0, 0,
+      sum_scaled_baseline_contribution_across_samples)
+  adjusted_sum_scaled_prediction_across_samples = adjusted_sum_scaled_baseline_contribution_across_samples + sum_scaled_media_contribution_across_channels_samples
+
+  # Calculate the media and baseline pct.
+  # Media/baseline contribution across samples/total prediction across samples.
+  media_contribution_pct_by_channel = (
+      sum_scaled_media_contribution_across_samples /
+      adjusted_sum_scaled_prediction_across_samples.reshape(-1, 1))
+  baseline_contribution_pct = adjusted_sum_scaled_baseline_contribution_across_samples / adjusted_sum_scaled_prediction_across_samples
+
+  # If the channel_names is none, then create naming covention for the channels.
+  if channel_names is None:
+    channel_names = media_mix_model.media_names
+
+  # Create media/baseline contribution pct as dataframes.
+  media_contribution_pct_by_channel_df = pd.DataFrame(
+      media_contribution_pct_by_channel, columns=channel_names)
+  baseline_contribution_pct_df = pd.DataFrame(
+      baseline_contribution_pct, columns=["baseline"])
+  contribution_pct_df = pd.merge(
+      media_contribution_pct_by_channel_df,
+      baseline_contribution_pct_df,
+      left_index=True,
+      right_index=True)
+
+  # If there's target scaler then inverse transform the posterior prediction.
+  posterior_pred = media_mix_model.trace["mu"]
+  if target_scaler:
+    posterior_pred = target_scaler.inverse_transform(posterior_pred)
+
+  # Take the sum of posterior predictions across geos.
+  if media_mix_model.trace["media_transformed"].ndim > 3:
+    posterior_pred = posterior_pred.sum(axis=-1)
+
+  # Take the average of the inverse transformed prediction across samples.
+  posterior_pred_df = pd.DataFrame(
+      posterior_pred.mean(axis=0), columns=["avg_prediction"])
+  contribution_pct_df.columns = [
+      "{}_percentage".format(col) for col in contribution_pct_df.columns
+  ]
+  contribution_df = pd.merge(
+      contribution_pct_df, posterior_pred_df, left_index=True, right_index=True)
+
+  # Create contribution by multiplying average prediction by media/baseline pct.
+  for channel in channel_names:
+    channel_contribution_col_name = "{} contribution".format(channel)
+    channel_pct_col = "{}_percentage".format(channel)
+    contribution_df.loc[:, channel_contribution_col_name] = contribution_df[
+        channel_pct_col] * contribution_df["avg_prediction"]
+    contribution_df.loc[:, channel_contribution_col_name] = contribution_df[
+        channel_contribution_col_name].astype("float")
+  contribution_df.loc[:, "baseline contribution"] = contribution_df[
+      "baseline_percentage"] * contribution_df["avg_prediction"]
+
+  period = np.arange(1, contribution_df.shape[0] + 1)
+  contribution_df.loc[:, "period"] = period
+  return contribution_df
+
+
 def plot_response_curves(
     media_mix_model: lightweight_mmm.LightweightMMM,
     media_scaler: Optional[preprocessing.CustomScaler] = None,
@@ -279,13 +405,13 @@ def plot_response_curves(
         x=media_ranges[:, i],
         y=predictions[:, i],
         label=media_mix_model.media_names[i],
-        color=sns.color_palette()[i],
+        color=_PALETTE[i],
         ax=ax)
     sns.lineplot(
         x=media_ranges[:, i],
         y=jnp.log(predictions[:, i]) if apply_log_scale else predictions[:, i],
         label=media_mix_model.media_names[i],
-        color=sns.color_palette()[i],
+        color=_PALETTE[i],
         ax=last_ax)
     if optimal_allocation_per_timeunit is not None:
       ax.plot(
@@ -294,14 +420,14 @@ def plot_response_curves(
           marker="o",
           markersize=marker_size,
           label="avg_spend",
-          color=sns.color_palette()[i])
+          color=_PALETTE[i])
       ax.plot(
           optimal_allocation_per_timeunit[i],
           optimal_allocation_predictions[i],
           marker="x",
           markersize=marker_size + 2,
           label="optimal_spend",
-          color=sns.color_palette()[i])
+          color=_PALETTE[i])
     ax.set_ylabel(kpi_label)
     ax.set_xlabel("Normalized Spend" if not media_scaler else "Spend")
     ax.legend(fontsize=legend_fontsize)
@@ -537,7 +663,7 @@ def plot_media_channel_posteriors(
     channel_names: Optional[Sequence[Any]] = None,
     quantiles: Sequence[float] = (0.05, 0.5, 0.95),
     fig_size: Optional[Tuple[int, int]] = None) -> matplotlib.figure.Figure:
-  """Plots the posterior distributions of estimated media channel effects.
+  """Plots the posterior distributions of estimated media channel effect.
 
   Model needs to be fit before calling this function to plot.
 
@@ -556,14 +682,14 @@ def plot_media_channel_posteriors(
     raise lightweight_mmm.NotFittedModelError(
         "Model needs to be fit first before attempting to plot its fit.")
 
-  n_media_channels = np.shape(media_mix_model.trace["beta_media"])[1]
+  n_media_channels = np.shape(media_mix_model.trace["coef_media"])[1]
   n_geos = (
       media_mix_model.media.shape[2] if media_mix_model.media.ndim == 3 else 1)
 
   if not fig_size:
     fig_size = (5 * n_geos, 3 * n_media_channels)
 
-  media_channel_posteriors = media_mix_model.trace["beta_media"]
+  media_channel_posteriors = media_mix_model.trace["coef_media"]
   if channel_names is None:
     channel_names = np.arange(np.shape(media_channel_posteriors)[1])
   fig, axes = plt.subplots(
@@ -601,8 +727,9 @@ def plot_bars_media_metrics(
 
   Args:
     metric: Estimated media metric as returned by
-      lightweight_mmm.get_posterior_metrics(). Can be either effects or ROI.
-    metric_name: Name of the media metric, e.g. effect or ROI.
+      lightweight_mmm.get_posterior_metrics(). Can be either contribution
+      percentage or ROI.
+    metric_name: Name of the media metric, e.g. contribution percentage or ROI.
     channel_names: Names of media channels to be added to plot.
     interval_mid_range: Mid range interval to take for plotting. Eg. .9 will use
       .05 and .95 as the lower and upper quantiles. Must be a float number.
@@ -640,3 +767,178 @@ def plot_bars_media_metrics(
   )
   plt.close()
   return fig
+
+
+def plot_pre_post_budget_allocation_comparison(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    kpi_with_optim: jnp.ndarray,
+    kpi_without_optim: jnp.ndarray,
+    optimal_buget_allocation: jnp.ndarray,
+    previous_budget_allocation: jnp.ndarray,
+    channel_names: Optional[Sequence[Any]] = None,
+    figure_size: Tuple[int, int] = (20, 10)
+) -> matplotlib.figure.Figure:
+  """Plots a barcharts to compare pre & post budget allocation.
+
+  Args:
+    media_mix_model: Media mix model to use for the optimization.
+    kpi_with_optim: Negative predicted target variable with optimized budget
+      allocation.
+    kpi_without_optim: negative predicted target variable with original budget
+      allocation proportion base on the historical data.
+    optimal_buget_allocation: Optmized budget allocation.
+    previous_budget_allocation: Starting budget allocation based on original
+      budget allocation proportion.
+    channel_names: Names of media channels to be added to plot.
+    figure_size: size of the plot.
+
+  Returns:
+    Barplots of budget allocation across media channels pre & post optimization.
+  """
+
+  if not hasattr(media_mix_model, "trace"):
+    raise lightweight_mmm.NotFittedModelError(
+        "Model needs to be fit first before attempting to plot its fit.")
+
+  previous_budget_allocation_pct = previous_budget_allocation / jnp.sum(
+      previous_budget_allocation)
+  optimized_budget_allocation_pct = optimal_buget_allocation / jnp.sum(
+      optimal_buget_allocation)
+
+  if channel_names is None:
+    channel_names = media_mix_model.media_names
+  x_axis = np.arange(len(channel_names))
+
+  pre_optimizaiton_predicted_target = kpi_without_optim * -1
+  post_optimization_predictiond_target = kpi_with_optim * -1
+  predictions = [
+      pre_optimizaiton_predicted_target, post_optimization_predictiond_target
+  ]
+
+  # Create bar chart.
+  fig, axes = plt.subplots(2, 1, figsize=figure_size)
+
+  plots1 = axes[0].bar(
+      x_axis - 0.2,
+      previous_budget_allocation,
+      width=0.4,
+      label="previous budget allocation")
+  plots2 = axes[0].bar(
+      x_axis + 0.2,
+      optimal_buget_allocation,
+      width=0.4,
+      label="optmized buget allocation")
+  axes[0].set_ylabel("Budget Allocation", fontsize="x-large")
+  axes[0].set_title(
+      "Before and After Optimization Budget Allocation Comparison",
+      fontsize="x-large")
+  # Iterrating over the bars one-by-one.
+  for bar_i in range(len(plots1.patches)):
+    bar = plots1.patches[bar_i]
+    axes[0].annotate(
+        "{:.0%}".format(previous_budget_allocation_pct[bar_i]),
+        (bar.get_x() + bar.get_width() / 2, bar.get_height()),
+        ha="center",
+        va="center",
+        size=10,
+        xytext=(0, 8),
+        textcoords="offset points")
+
+  # Iterrating over the bars one-by-one.
+  for bar_i in range(len(plots2.patches)):
+    bar = plots2.patches[bar_i]
+    axes[0].annotate(
+        "{:.0%}".format(optimized_budget_allocation_pct[bar_i]),
+        (bar.get_x() + bar.get_width() / 2, bar.get_height()),
+        ha="center",
+        va="center",
+        size=10,
+        xytext=(0, 8),
+        textcoords="offset points")
+
+  axes[0].set_xticks(x_axis)
+  axes[0].set_xticklabels(channel_names, fontsize="medium")
+  axes[0].legend(fontsize="medium")
+
+  plots3 = axes[1].bar([
+      "pre optimization predictited target",
+      "post optimization predicted target"
+  ], predictions)
+  axes[1].set_ylim(
+      min(predictions) - min(predictions) * 0.1,
+      max(predictions) + min(predictions) * 0.1)
+  axes[1].set_ylabel("Predicted Target Variable", fontsize="x-large")
+  axes[1].set_title(
+      "Pre Post Optimization Target Variable Comparison", fontsize="x-large")
+  axes[1].set_xticks(range(2))
+  axes[1].set_xticklabels([
+      "pre optimization predictited target",
+      "post optimization predicted target"
+  ],
+                          fontsize="x-large")
+
+  # Iterrating over the bars one-by-one.
+  for bar_i in range(len(plots3.patches)):
+    bar = plots3.patches[bar_i]
+    axes[1].annotate(
+        "{:,.1f}".format(predictions[bar_i]),
+        (bar.get_x() + bar.get_width() / 2, bar.get_height()),
+        ha="center",
+        va="center",
+        size=10,
+        xytext=(0, 8),
+        textcoords="offset points")
+
+  plt.tight_layout()
+  plt.close()
+  return fig
+
+
+def plot_media_baseline_contribution_area_plot(
+    media_mix_model: lightweight_mmm.LightweightMMM,
+    target_scaler: Optional[preprocessing.CustomScaler] = None,
+    channel_names: Optional[Sequence[Any]] = None,
+    fig_size: Optional[Tuple[int, int]] = (30, 10)
+) -> matplotlib.figure.Figure:
+  """Plots an area chart to visualize weekly media & baseline contribution.
+
+  Args:
+    media_mix_model: Media mix model.
+    target_scaler: Scaler used for scaling the target.
+    channel_names: Names of media channels.
+    fig_size: Size of the figure to plot as used by matplotlib.
+
+  Returns:
+    Stacked area chart of weekly baseline & media contribution.
+  """
+  # Create media channels & baseline contribution dataframe.
+  contribution_df = create_media_baseline_contribution_df(
+      media_mix_model=media_mix_model,
+      target_scaler=target_scaler,
+      channel_names=channel_names)
+
+  # Create contribution dataframe for the plot.
+  contribution_columns = [
+      col for col in contribution_df.columns if "contribution" in col
+  ]
+  contribution_df_for_plot = contribution_df.loc[:, contribution_columns]
+  contribution_df_for_plot = contribution_df_for_plot[
+      contribution_df_for_plot.columns[::-1]]
+  period = np.arange(1, contribution_df_for_plot.shape[0] + 1)
+  contribution_df_for_plot.loc[:, "period"] = period
+
+  # Plot the stacked area chart.
+  fig, ax = plt.subplots()
+  contribution_df_for_plot.plot.area(
+      x="period", stacked=True, figsize=fig_size, ax=ax)
+  ax.set_title("Attribution Over Time", fontsize="x-large")
+  ax.tick_params(axis="y")
+  ax.set_ylabel("Baseline & Media Chanels Attribution")
+  ax.set_xlabel("Period")
+  ax.set_xlim(1, contribution_df_for_plot["period"].max())
+  ax.set_xticks(contribution_df_for_plot["period"])
+  ax.set_xticklabels(contribution_df_for_plot["period"])
+  for tick in ax.get_xticklabels():
+    tick.set_rotation(45)
+  plt.close()
+  return fig
